@@ -1,13 +1,14 @@
-import re
-import time
+# warnings.filterwarnings('ignore', message='Unverified HTTPS request')
 import json
+import os
+import re
+from datetime import datetime
+from time import time
 import jinja2
-
+from requests import exceptions
 from src.edc_utilities import edc_session_helper
 from src.metadata_utilities import messages, generic, generic_settings, mu_logging
 
-# warnings.filterwarnings('ignore', message='Unverified HTTPS request')
-
 
 class EDCLineage:
     """
@@ -16,6 +17,10 @@ class EDCLineage:
 
     code_version = "0.3.11"
     total = 10000
+    this_run = datetime.now().isoformat(timespec="microseconds").replace(":", "-")
+    this_run_dir = "out/" + this_run
+    os.makedirs(this_run_dir, exist_ok=True)
+    patch_payload_file = this_run_dir + "/patch_payloads.txt"
 
     def __init__(self, settings, mu_log_ref):
         self.offset = 0
@@ -33,20 +38,15 @@ class EDCLineage:
                 self.mu_log = self.settings.mu_log
         else:
             self.mu_log = mu_log_ref
-        self.generic = None
+        self.generic = generic.Generic(settings=self.settings, mu_log_ref=self.mu_log)
         self.edc_source_resource_name = "unknown"
         self.edc_source_datasource = "unknown"
         self.edc_source_folder = "unknown"
-        self.edc_target_filesystem = "unknown"
+        self.edc_target_resource_name = "unknown"
         self.edc_target_datasource = "unknown"
         self.edc_target_folder = "unknown"
-        self.head = {'Content-Type': 'application/json'}
+        # self.head = {'Content-Type': 'application/json'}
         self.edc_helper = edc_session_helper.EDCSession(self.settings)
-        self.proxies = "unknown"
-        self.jinja_base_directory = None
-        self.jinja_application = None
-        self.jinja_templates = None
-        self.jinja_template_directory = None
         self.meta_type = "unknown"
         self.data = None
         self.template = None
@@ -55,7 +55,7 @@ class EDCLineage:
         self.edc_source_resource_name = self.settings.edc_config_data["edc_source_resource_name"]
         self.edc_source_datasource = self.settings.edc_config_data["edc_source_datasource"]
         self.edc_source_folder = self.settings.edc_config_data["edc_source_container"]
-        self.edc_target_filesystem = self.settings.edc_config_data["edc_target_resource_name"]
+        self.edc_target_resource_name = self.settings.edc_config_data["edc_target_resource_name"]
         self.edc_target_datasource = self.settings.edc_config_data["edc_target_datasource"]
         self.edc_target_folder = self.settings.edc_config_data["edc_target_container"]
 
@@ -63,7 +63,7 @@ class EDCLineage:
         self.edc_source_resource_name = self.settings.edc_config_data["edc_source_filesystem"]
         self.edc_source_datasource = self.settings.edc_config_data["edc_source_datasource"]
         self.edc_source_folder = self.settings.edc_config_data["edc_source_folder"]
-        self.edc_target_filesystem = self.settings.edc_config_data["edc_target_filesystem"]
+        self.edc_target_resource_name = self.settings.edc_config_data["edc_target_filesystem"]
         self.edc_target_datasource = self.settings.edc_config_data["edc_target_datasource"]
         self.edc_target_folder = self.settings.edc_config_data["edc_target_folder"]
 
@@ -103,21 +103,6 @@ class EDCLineage:
             self.mu_log.log(self.mu_log.DEBUG, "Generic settings already loaded.", module)
             self.settings = generic_settings
 
-        self.generic = generic.Generic(settings=self.settings, mu_log_ref=self.mu_log)
-        self.proxies = self.settings.get_edc_proxy()
-
-        # Get jinja configuration
-        if self.settings.jinja_config is None:
-            result = self.settings.get_config()
-            self.mu_log.log(self.mu_log.DEBUG, "settings.get_config returned: " + result["code"], module)
-
-        result = self.generic.get_jinja_settings()
-        if result == messages.message["ok"]:
-            self.mu_log.log(self.mu_log.INFO, "Jinja configuration file found and read.", module)
-        else:
-            self.mu_log.log(self.mu_log.ERROR, "get_jinja_settings returned: " + result["code"], module)
-            return result
-
         result = self.get_edc_data_references()
         if result == messages.message["ok"]:
             self.mu_log.log(self.mu_log.DEBUG, "EDC Data references for scanned objects determined.", module)
@@ -150,7 +135,7 @@ class EDCLineage:
             self.mu_log.log(self.mu_log.DEBUG, "Found jinja template: " + self.meta_type + ".json", module)
         except jinja2.exceptions.TemplateNotFound:
             self.mu_log.log(self.mu_log.ERROR, "Could not find jinja template >" + self.meta_type + ".json<."
-                            + " in directory >" + self.jinja_template_directory + "<."
+                            + " in directory >" + self.generic.jinja_template_directory + "<."
                             , module)
             return messages.message["jinja_template_not_found"], None
 
@@ -215,7 +200,7 @@ class EDCLineage:
                 return build_result, "{}"
 
             target_name = self.generic.found_data["name"]
-            target = self.edc_target_filesystem \
+            target = self.edc_target_resource_name \
                      + self.edc_target_datasource \
                      + self.edc_target_folder \
                      + target_name
@@ -280,7 +265,7 @@ class EDCLineage:
 
             to_entity_data = self.generic.found_data
             to_entity_name = to_entity_data["name"]
-            to_attribute = self.edc_target_filesystem \
+            to_attribute = self.edc_target_resource_name \
                            + self.edc_target_datasource \
                            + self.edc_target_folder \
                            + to_entity_name \
@@ -295,10 +280,11 @@ class EDCLineage:
             self.mu_log.log(self.mu_log.DEBUG, "number of entries in attribute list: " + str(len(from_attribute_list)),
                             module)
             if len(from_attribute_list) == 0:
-                new_entry = template_new_source_links.render(source_object_id=to_attribute,
-                                                             data_flow="core.DirectionalDataFlow")
-                self.mu_log.log(self.mu_log.VERBOSE, "new entry: " + new_entry, module)
-                source_target_list.append(new_entry)
+                self.mu_log.log(self.mu_log.INFO, "No source attributes for >" + to_attribute + "<.", module)
+                # new_entry = template_new_source_links.render(source_object_id=to_attribute,
+                #                                             data_flow="core.DirectionalDataFlow")
+                # self.mu_log.log(self.mu_log.VERBOSE, "new entry: " + new_entry, module)
+                # source_target_list.append(new_entry)
 
             i = 0
             for attribute in from_attribute_list:
@@ -349,10 +335,10 @@ class EDCLineage:
         the_entries = ",".join(update_entry_list)
         payload = template_updates.render(update_entries=the_entries)
         # TODO: Find a good solution for this
-        payload = payload.replace("<<NONE>>", self.edc_target_filesystem
-                                            + self.edc_target_datasource
-                                            + self.edc_target_folder
-                                            + to_entity_name)
+        payload = payload.replace("<<NONE>>", self.edc_target_resource_name
+                                  + self.edc_target_datasource
+                                  + self.edc_target_folder
+                                  + to_entity_name)
         self.mu_log.log(self.mu_log.VERBOSE, "payload: " + payload, module)
 
         return build_result, payload
@@ -360,12 +346,12 @@ class EDCLineage:
     def send_metadata_to_edc(self, suppress_edc_call=False, method="PATCH", uri="/access/1/catalog/data/objects"
                              , etag=None, parameters=None, payload=None):
         module = __name__ + ".send_metadata_to_edc"
-        start_time = time.time()
+        start_time = time()
 
         self.mu_log.log(self.mu_log.DEBUG, "EDC base URL: " + self.settings.edc_url, module)
         url = self.settings.edc_url + uri
         self.mu_log.log(self.mu_log.DEBUG, "Used URL >" + url + "<.", module)
-        self.mu_log.log(self.mu_log.VERBOSE, "Proxies: " + str(self.proxies), module)
+        self.mu_log.log(self.mu_log.VERBOSE, "Proxies (from session): " + str(self.edc_helper.session.proxies), module)
         if self.settings.edc_auth is None:
             self.mu_log.log(self.mu_log.WARNING
                             , "Auth not set. Add edc_auth to the edc.secrets file or set INFA_EDC_AUTH"
@@ -384,36 +370,73 @@ class EDCLineage:
         else:
             if method == "PATCH":
                 # re-init: self.edc_helper.session = self.edc_helper.init_edc_session()
-                self.mu_log.log(self.mu_log.VERBOSE, "sending PATCH payload >" + str(payload) + "<.", module)
-                self.mu_log.log(self.mu_log.VERBOSE, "PATCH Request Headers: " + str(self.head.items()), module)
-                self.mu_log.log(self.mu_log.VERBOSE, "Session PATCH Request headers: " + str(self.edc_helper.session.headers.items()), module)
-                response = self.edc_helper.session.patch(url, payload, timeout=self.settings.edc_timeout,
-                                                         proxies=self.proxies, headers=self.head)
-                self.mu_log.log(self.mu_log.VERBOSE, "PATCH Response headers: " + str(response.headers), module)
+                self.mu_log.log(self.mu_log.VERBOSE, "PATCH URL >" + url + "<.", module)
+                self.mu_log.log(self.mu_log.VERBOSE, "PATCH payload >" + payload + "<.", module)
+                self.mu_log.log(self.mu_log.VERBOSE,
+                                "sending PATCH payload (as data object) >" + str(payload) + "<.", module)
+                self.mu_log.log(self.mu_log.VERBOSE, "Session PATCH Request headers: "
+                                + str(self.edc_helper.session.headers.items()), module)
+                try:
+                    with open(self.patch_payload_file, "a") as payloads:
+                        payloads.write("##START##\n")
+                        payloads.write(payload + "\n")
+                        payloads.write("##END##\n")
+                except IOError:
+                    self.mu_log.log(self.mu_log.WARNING, "Could not write payload to payloads file >"
+                                    + self.patch_payload_file + "<", module)
+                try:
+                    # https://requests.readthedocs.io/en/master/_modules/requests/sessions/
+                    # patch uses data=, although json= can be part of **kwargs
+                    response = self.edc_helper.session.patch(url
+                                                             , data=payload
+                                                             )
+                    self.mu_log.log(self.mu_log.VERBOSE, "PATCH Response headers: " + str(response.headers), module)
+                except exceptions.ConnectTimeout:
+                    self.mu_log.log(self.mu_log.ERROR, "Connection to EDC failed due to a timeout.", module)
+                    response = None
+                except exceptions.ConnectionError:
+                    self.mu_log.log(self.mu_log.ERROR, "Connection error connecting to EDC.", module)
+                    response = None
+
             elif method == "PUT":
                 if etag is None:
                     self.mu_log.log(self.mu_log.WARNING, "eTag is None. This should not happen.", module)
                 else:
-                    self.head.update({"If-Match": etag})
+                    self.edc_helper.session.headers.update({"If-Match": etag})
                 self.mu_log.log(self.mu_log.VERBOSE, "sending PUT payload >" + str(payload) + "<.", module)
-                self.mu_log.log(self.mu_log.VERBOSE, "PUT Request Headers: " + str(self.head.items()), module)
-                response = self.edc_helper.session.put(url=url, data=payload, timeout=self.settings.edc_timeout,
-                                                       headers=self.head,
-                                                       proxies=self.proxies)
-                self.mu_log.log(self.mu_log.VERBOSE, "PUT Response headers: " + str(response.headers), module)
+                self.mu_log.log(self.mu_log.VERBOSE, "PUT Request Headers: "
+                                + str(self.edc_helper.session.headers.items()), module)
+                try:
+                    response = self.edc_helper.session.put(url=url, data=payload)
+                    self.mu_log.log(self.mu_log.VERBOSE, "PUT Response headers: " + str(response.headers), module)
+                except exceptions.ConnectTimeout:
+                    self.mu_log.log(self.mu_log.ERROR, "Connection to EDC failed due to a timeout.", module)
+                    response = None
+                except exceptions.ConnectionError:
+                    self.mu_log.log(self.mu_log.ERROR, "Connection error connecting to EDC.", module)
+                    response = None
             elif method == "GET":
-                self.mu_log.log(self.mu_log.VERBOSE, "GET Request Headers: " + str(self.head.items())
+                self.mu_log.log(self.mu_log.VERBOSE, "GET Request Headers: "
+                                + str(self.edc_helper.session.headers.items())
                                 , module)
-                response = self.edc_helper.session.get(url, timeout=self.settings.edc_timeout,
-                                                       proxies=self.proxies,
-                                                       headers=self.head,
-                                                       params=parameters)
-                self.mu_log.log(self.mu_log.VERBOSE, "GET Response headers: " + str(response.headers), module)
+                try:
+                    response = self.edc_helper.session.get(url,
+                                                           params=parameters)
+                    self.mu_log.log(self.mu_log.VERBOSE, "GET Response headers: " + str(response.headers), module)
+                except exceptions.ConnectTimeout:
+                    self.mu_log.log(self.mu_log.ERROR, "Connection to EDC failed due to a timeout.", module)
+                    response = None
+                except exceptions.ConnectionError:
+                    self.mu_log.log(self.mu_log.ERROR, "Connection error connecting to EDC.", module)
+                    response = None
             else:
                 self.mu_log.log(self.mu_log.ERROR, "Invalid HTTP Method in call. Internal error.", module)
                 return messages.message["invalid_http_method"]
 
             # TODO: Success is not always 200, but maybe for Informatica API calls it is...
+            if response is None:
+                return messages.message["edc_connection_failed"], None
+
             status = response.status_code
             if status != 200:
                 # some error - e.g. catalog not running, or bad credentials
@@ -428,7 +451,7 @@ class EDCLineage:
                     self.mu_log.log(self.mu_log.ERROR, "EDC API did not return a JSON payload", module)
                     send_result = messages.message["invalid_api_response"]
 
-        run_time = time.time() - start_time
+        run_time = time() - start_time
         self.mu_log.log(self.mu_log.DEBUG,
                         "send to EDC completed with " + send_result["code"] + ". run time: " + str(run_time), module)
         return send_result, response
@@ -598,6 +621,7 @@ class EDCLineage:
         attribute_count = 0
         found_attribute = False
         attribute_id = None
+        json_response = None
 
         while offset < total and not found_attribute:
             page += 1
@@ -608,14 +632,15 @@ class EDCLineage:
                                                          , method="GET"
                                                          , uri="/access/2/catalog/models/attributes"
                                                          , parameters=paging_params)
-            json_response = response.json()
+
             if result == messages.message["ok"]:
+                json_response = response.json()
                 self.mu_log.log(self.mu_log.VERBOSE, "Call to EDC returned OK.", module)
             else:
                 self.mu_log.log(self.mu_log.ERROR, "Call to EDC returned: " + result["code"]
-                                + " - " + json_response["message"]
                                 , module)
-                return messages.message["custom_attribute_not_found"], None
+                return result, None
+
             total = json_response["metadata"]["totalCount"]
             self.mu_log.log(self.mu_log.VERBOSE, "There are >" + str(total) + "< attributes in the response."
                             , module)
@@ -711,11 +736,11 @@ class EDCLineage:
 
         to_entity_data = self.generic.found_data
         to_entity_name = to_entity_data["name"]
-        to_entity_id = self.edc_target_filesystem \
+        to_entity_id = self.edc_target_resource_name \
                        + self.edc_target_datasource \
                        + self.edc_target_folder \
                        + to_entity_name
-        to_attribute_id = self.edc_target_filesystem \
+        to_attribute_id = self.edc_target_resource_name \
                           + self.edc_target_datasource \
                           + self.edc_target_folder \
                           + to_entity_name \
